@@ -296,9 +296,9 @@ class OpenIDRestClient(RestClient):
     Args:
         address (str): base address of REST API
         token_url (str): base address of token service
-        refresh_token (str): initial refresh token
         client_id (str): client id
-        client_secret (str): client secret (optional - required for refresh tokens)
+        client_secret (str): client secret (optional - required to generate new refresh token)
+        refresh_token (str): initial refresh token (optional)
         update_func (callable): a function that gets called when the access and refresh tokens are updated (optional)
         timeout (int): request timeout (optional)
         retries (int): number of retries to attempt (optional)
@@ -307,19 +307,22 @@ class OpenIDRestClient(RestClient):
         self,
         address: str,
         token_url: str,
-        refresh_token: str,
         client_id: str,
         client_secret: Optional[str] = None,
+        refresh_token: Optional[Union[str, bytes]] = None,
         update_func: Optional[Callable[[Union[str, bytes], Optional[Union[str, bytes]]], None]] = None,
         **kwargs: Any
     ) -> None:
+        if (not client_secret) and (not refresh_token):
+            raise RuntimeError('Either client_secret or refresh_token is required')
+
         super().__init__(address, **kwargs)
         self.auth = OpenIDAuth(token_url)
         self.client_id = client_id
         self.client_secret = client_secret
 
         self.access_token = None
-        self.refresh_token: Optional[Union[str, bytes]] = refresh_token
+        self.refresh_token = refresh_token
         self.token_func = True  # type: ignore
         self.update_func = update_func
         self._get_token()
@@ -360,4 +363,26 @@ class OpenIDRestClient(RestClient):
                     self.update_func(self.access_token, self.refresh_token)
                 return
 
+        if not self.refresh_token and self.client_secret:
+            # try making a new token
+            args = {
+                'grant_type': 'client_credentials',
+                'client_id': self.client_id,
+                'client_secret': self.client_secret,
+            }
+
+            try:
+                r = requests.post(self.auth.token_url, data=args)
+                r.raise_for_status()
+                req = r.json()
+            except Exception:
+                self.refresh_token = None
+            else:
+                self.logger.debug('OpenID token created')
+                self.access_token = req['access_token']
+                self.refresh_token = req['refresh_token'] if 'refresh_token' in req else None
+                if self.access_token and self.update_func:
+                    self.update_func(self.access_token, self.refresh_token)
+                return
+
         raise Exception('No token available')
